@@ -316,6 +316,7 @@ struct ConnectionClass_
 	char		result_uncommitted;
 	char		lo_is_domain;
 	char		*original_client_encoding;
+	char		*locale_encoding;
 	char		*server_encoding;
 	Int2		ccsc;
 	Int2		mb_maxbyte_per_char;
@@ -441,6 +442,7 @@ void	CC_examine_global_transaction(ConnectionClass *self);
 
 
 BOOL CC_set_transact(ConnectionClass *self, UInt4 isolation);
+int CC_send_client_encoding(ConnectionClass *self, const char *encoding);
 
 /* CC_send_query options */
 enum {
