@@ -22,10 +22,8 @@
 #include "misc.h" // strncpy_null
 #include "win_setup.h"
 
-#include "convert.h"
 #include "loadlib.h"
 
-#include "multibyte.h"
 #include "pgapifunc.h"
 #ifdef _HANDLE_ENLIST_IN_DTC_
 #include "xalibname.h"
@@ -337,6 +335,27 @@ mylog("GetProcAddres for %s\n", procname);
 }
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 
+#include <sys/stat.h>
+static const char *IsAnExistingDirectory(const char *path)
+{
+		
+	struct stat st;
+
+	if (stat(path, &st) < 0)
+	{
+		CSTR errmsg_doesnt_exist = "doesn't exist";
+
+		return errmsg_doesnt_exist;
+	}
+	if ((st.st_mode & S_IFDIR) == 0)
+	{
+		CSTR errmsg_isnt_a_dir = "isn't a directory";
+		
+		return errmsg_isnt_a_dir;
+	}
+	return NULL;
+}
+
 LRESULT			CALLBACK
 global_optionsProc(HWND hdlg,
 				   UINT wMsg,
@@ -349,6 +368,7 @@ global_optionsProc(HWND hdlg,
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 	ConnInfo	*ci;
 	char logdir[PATH_MAX];
+	const char *logmsg;
 
 	switch (wMsg)
 	{
@@ -379,11 +399,16 @@ global_optionsProc(HWND hdlg,
 			switch (GET_WM_COMMAND_ID(wParam, lParam))
 			{
 				case IDOK:
+					GetDlgItemText(hdlg, DS_LOGDIR, logdir, sizeof(logdir));
+					if (logdir[0] && (logmsg = IsAnExistingDirectory(logdir)) != NULL)
+					{
+						MessageBox(hdlg, "Folder for logging error", logmsg, MB_ICONEXCLAMATION | MB_OK);
+						break;
+					}
 					globals.commlog = IsDlgButtonChecked(hdlg, DRV_COMMLOG);
 					globals.debug = IsDlgButtonChecked(hdlg, DRV_DEBUG);
-					if (writeDriverCommoninfo(ODBCINST_INI, NULL, &globals) < 0)
+					if (writeDriverCommoninfo(ODBCINST_INI, ci->drivername, &globals) < 0)
 						MessageBox(hdlg, "Sorry, impossible to update the values\nWrite permission seems to be needed", "Update Error", MB_ICONEXCLAMATION | MB_OK);
-					GetDlgItemText(hdlg, DS_LOGDIR, logdir, sizeof(logdir));
 					setLogDir(logdir[0] ? logdir : NULL);
 #ifdef _HANDLE_ENLIST_IN_DTC_
 					hmodule = DtcProc("SetMsdtclog", &proc);
@@ -752,15 +777,51 @@ ds_options3Draw(HWND hdlg, const ConnInfo *ci)
 		EnableWindow(GetDlgItem(hdlg, DS_DTC_SIMPLE_PRECHECK), enable);
 		EnableWindow(GetDlgItem(hdlg, DS_DTC_CONFIRM_RM_CONNECTION), enable);
 	}
+	/* Datasource libpq parameters */
+	SetDlgItemText(hdlg, DS_LIBPQOPT, SAFE_NAME(ci->pqopt));
 
 	return 0;
 }
 
+void *PQconninfoParse(const char *, char **);
+void PQconninfoFree(void *);
+typedef void *(*PQCONNINFOPARSEPROC)(const char *, char **);
+typedef void (*PQCONNINFOFREEPROC)(void *); 
 static int
 ds_options3_update(HWND hdlg, ConnInfo *ci)
 {
+	void	*connOpt = NULL;
+	char	*ermsg = NULL;
+	char	pqopt[LARGE_REGISTRY_LEN];
+	HMODULE	hmodule;
+	PQCONNINFOPARSEPROC	pproc = NULL;
+	PQCONNINFOFREEPROC	fproc = NULL;	
+
 	mylog("%s: got ci = %p\n", __FUNCTION__, ci);
 
+	/* Datasource libpq parameters */
+	GetDlgItemText(hdlg, DS_LIBPQOPT, pqopt, sizeof(pqopt));
+	if (hmodule = LoadLibraryEx("libpq.dll", NULL, LOAD_WITH_ALTERED_SEARCH_PATH), NULL != hmodule)
+	{
+		pproc = (PQCONNINFOPARSEPROC) GetProcAddress(hmodule, "PQconninfoParse");
+		if (NULL != pproc && NULL == (connOpt= (*pproc)(pqopt, &ermsg)))
+		{
+			const char *logmsg = "libpq parameter error";
+
+			MessageBox(hdlg, ermsg ? ermsg : "memory over?", logmsg, MB_ICONEXCLAMATION | MB_OK);
+			if (NULL != ermsg)
+				free(ermsg);
+			FreeLibrary(hmodule);
+
+			return 1;
+		}
+		fproc = (PQCONNINFOFREEPROC) GetProcAddress(hmodule, "PQconninfoFree");
+		if (fproc)
+			(*fproc)(connOpt);
+		FreeLibrary(hmodule);
+	}
+	STR_TO_NAME(ci->pqopt, pqopt);
+
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	if (IsDlgButtonChecked(hdlg, DS_DTC_LINK_ONLY))
 		ci->xa_opt = DTC_CHECK_LINK_ONLY;
