@@ -16,6 +16,9 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef	WIN32
+#include <locale.h>
+#endif
 #ifndef	TRUE
 #define	TRUE	1
 #endif
@@ -79,6 +82,7 @@ static pg_CS CS_Alias[] =
 	{ "TCVN",	WIN1258 },
 	{ "ALT",	WIN866 },
 	{ "WIN",	WIN1251 },
+	{ "KOI8R",	KOI8R },
 	{ "OTHER",	OTHER }
 };
 
@@ -161,15 +165,20 @@ check_client_encoding(const pgNAME conn_settings)
 				}
 				step++;
 				cptr += 15;
+				if ('=' == *cptr)
+					cptr--;
 				break;
 			case 2:
-				if (0 != strnicmp(cptr, "to", 2))
+				if (0 == strnicmp(cptr, "to", 2))
+					cptr += 2;
+				else if (0 == strnicmp(cptr, "=", 1))
+					;
+				else
 				{
 					allowed_cmd = FALSE;
 					continue;
 				}
 				step++;
-				cptr += 2;
 				break;
 			case 3:
 				if (LITERAL_QUOTE == *cptr)
@@ -179,9 +188,11 @@ check_client_encoding(const pgNAME conn_settings)
 				}
 				else
 				{
-					for (sptr = cptr; *cptr && !isspace((unsigned char) *cptr); cptr++) ;
+					for (sptr = cptr; *cptr && ';' != *cptr && !isspace((unsigned char) *cptr); cptr++) ;
 				}
 				len = cptr - sptr;
+				if (';' == *cptr)
+					cptr--;
 				step++;
 				break;
 		}
@@ -197,13 +208,13 @@ check_client_encoding(const pgNAME conn_settings)
 	return rptr;
 }
 
-static int
+int
 pg_mb_maxlen(int characterset_code)
 {
 	switch (characterset_code)
 	{
 		case UTF8:
-			return 6;
+			return 4;
 		case EUC_TW:
 			return 4;
 		case EUC_JIS_2004:
@@ -413,27 +424,26 @@ pg_CS_stat(int stat,unsigned int character,int characterset_code)
 }
 
 /*
- *	This function works under Windows or Unicode case only.
- *	Simply returns NULL under other OSs.
+ *	This function is used to know the encoding corresponding to
+ *	the current locale.
  */
-#ifndef	UNICODE_SUPPORT
-static const char *
-get_environment_encoding(const ConnectionClass *conn, const char *setenc, const char *currenc)
+const char *
+derive_locale_encoding(const char *dbencoding)
 {
 	const char *wenc = NULL;
 #ifdef	WIN32
 	int	acp;
+#else
+	const char *loc, *ptr;
 #endif /* WIN32 */
 
-	if (setenc && stricmp(setenc, "OTHER"))
-		return setenc;
-	if (wenc = getenv("PGCLIENTENCODING"), NULL != wenc)
+	if (wenc = getenv("PGCLIENTENCODING"), NULL != wenc) /* environmnt variable */
 		return wenc;
 #ifdef	WIN32
 	acp = GetACP();
 	if (acp >= 1251 && acp <= 1258)
 	{
-		if (stricmp(currenc, "SQL_ASCII") == 0)
+		if (stricmp(dbencoding, "SQL_ASCII") == 0)
 			return wenc;
 	}
 	switch (acp)
@@ -460,107 +470,43 @@ get_environment_encoding(const ConnectionClass *conn, const char *setenc, const
 			wenc = "WIN1256";
 			break;
 		case 1252:
-			if (strnicmp(currenc, "LATIN", 5) == 0)
+			if (strnicmp(dbencoding, "LATIN", 5) == 0)
 				break;
-			if (PG_VERSION_GE(conn, 8.1))
-				wenc = "WIN1252";
-			else
-				wenc = "LATIN1";
+			wenc = "WIN1252";
 			break;
 		case 1258:
-			if (PG_VERSION_GE(conn, 8.1))
-				wenc = "WIN1258";
+			wenc = "WIN1258";
 			break;
 		case 1253:
-			if (PG_VERSION_GE(conn, 8.2))
-				wenc = "WIN1253";
+			wenc = "WIN1253";
 			break;
 		case 1254:
-			if (PG_VERSION_GE(conn, 8.2))
-				wenc = "WIN1254";
+			wenc = "WIN1254";
 			break;
 		case 1255:
-			if (PG_VERSION_GE(conn, 8.2))
-				wenc = "WIN1255";
+			wenc = "WIN1255";
 			break;
 		case 1257:
-			if (PG_VERSION_GE(conn, 8.2))
-				wenc = "WIN1257";
+			wenc = "WIN1257";
 			break;
 	}
+#else
+	/*
+	 *	Derive the encoding from the codeset part of the current locale.
+	 */
+	loc = setlocale(LC_ALL, "");
+	if (loc && (ptr = strchr(loc, '.')))
+	{
+		int enc_no;
+
+		ptr++;
+		if ((enc_no= pg_char_to_encoding(ptr)) >= 0)
+			wenc = pg_encoding_to_char(enc_no);
+		mylog("%s locale=%s enc=%s\n", __FUNCTION__, loc, wenc ? wenc : "(null)");
+	}
 #endif /* WIN32 */
 	return wenc;
 }
-#endif /* !UNICODE_SUPPORT */
-
-void
-CC_lookup_characterset(ConnectionClass *self)
-{
-	const char *encspec, *currenc;
-	const char *tencstr;
-	CSTR func = "CC_lookup_characterset";
-
-	mylog("%s: entering...\n", func);
-	encspec = self->original_client_encoding;
-	currenc = PQparameterStatus(self->pqconn, "client_encoding");
-
-	tencstr = encspec ? encspec : currenc;
-	if (self->original_client_encoding)
-	{
-		if (stricmp(self->original_client_encoding, tencstr))
-		{
-			char msg[256];
-
-			snprintf(msg, sizeof(msg), "The client_encoding '%s' was changed to '%s'", self->original_client_encoding, tencstr);
-			CC_set_error(self, CONN_OPTION_VALUE_CHANGED, msg, func);
-		}
-		free(self->original_client_encoding);
-	}
-#ifndef	UNICODE_SUPPORT
-	else
-	{
-		const char *wenc = get_environment_encoding(self, encspec, currenc);
-		if (wenc && (!tencstr || stricmp(tencstr, wenc)))
-		{
-			QResultClass	*res;
-			char		query[64];
-			int		errnum = CC_get_errornumber(self);
-			BOOL		cmd_success;
-
-			sprintf(query, "set client_encoding to '%s'", wenc);
-			res = CC_send_query(self, query, NULL, IGNORE_ABORT_ON_CONN | ROLLBACK_ON_ERROR, NULL);
-			cmd_success = QR_command_maybe_successful(res);
-			QR_Destructor(res);
-			CC_set_errornumber(self, errnum);
-			if (cmd_success)
-			{
-				self->original_client_encoding = strdup(wenc);
-				self->ccsc = pg_CS_code(self->original_client_encoding);
-				return;
-			}
-		}
-	}
-#endif /* UNICODE_SUPPORT */
-	if (tencstr)
-	{
-		self->original_client_encoding = strdup(tencstr);
-		self->ccsc = pg_CS_code(tencstr);
-		qlog("    [ Client encoding = '%s' (code = %d) ]\n", self->original_client_encoding, self->ccsc);
-		if (self->ccsc < 0)
-		{
-			char msg[256];
-
-			snprintf(msg, sizeof(msg), "would handle the encoding '%s' like ASCII", tencstr);
-			CC_set_error(self, CONN_OPTION_VALUE_CHANGED, msg, func);
-		}
-	}
-	else
-	{
-		self->ccsc = SQL_ASCII;
-		self->original_client_encoding = NULL;
-	}
-	self->mb_maxbyte_per_char = pg_mb_maxlen(self->ccsc);
-}
 
 void encoded_str_constr(encoded_str *encstr, int ccsc, const char *str)
 {
