@@ -54,7 +54,6 @@ enum {
 	,OTHER		=	-1
 };
 
-extern void CC_lookup_characterset(ConnectionClass *self);
 extern int pg_CS_code(const char *stat_string);
 
 /* Old Type Compatible */
@@ -74,4 +73,6 @@ extern ssize_t encoded_position_shift(encoded_str *encstr, size_t shift);
 extern int encoded_byte_check(encoded_str *encstr, size_t abspos);
 /* #define check_client_encoding(X) pg_CS_name(pg_CS_code(X)) */
 char *check_client_encoding(const pgNAME sql_string);
+const char *derive_locale_encoding(const char *dbencoding);
+int pg_mb_maxlen(int characterset_code);
 #endif /* __MULTIBUYTE_H__ */
