@@ -90,6 +90,7 @@ static BOOL theResultIsEmpty(const StatementClass *stmt)
 	return (0 == QR_get_num_total_tuples(res));
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLColumns(HSTMT StatementHandle,
 		   SQLCHAR *CatalogName, SQLSMALLINT NameLength1,
@@ -102,6 +103,7 @@ SQLColumns(HSTMT StatementHandle,
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	SQLCHAR	*ctName = CatalogName, *scName = SchemaName,
 		*tbName = TableName, *clName = ColumnName;
+	ConnInfo *ci = &(SC_get_conn(stmt)->connInfo);
 	UWORD	flag	= PODBC_SEARCH_PUBLIC_SCHEMA;
 
 	mylog("[%s]", func);
@@ -113,6 +115,10 @@ SQLColumns(HSTMT StatementHandle,
 	StartRollbackState(stmt);
 	if (stmt->options.metadata_id)
 		flag |= PODBC_NOT_SEARCH_PATTERN;
+	if (atoi(ci->show_oid_column))
+		flag |= PODBC_SHOW_OID_COLUMN;
+	if (atoi(ci->row_versioning))
+		flag |= PODBC_ROW_VERSIONING;
 	if (SC_opencheck(stmt, func))
 		ret = SQL_ERROR;
 	else
@@ -168,7 +174,6 @@ SQLColumns(HSTMT StatementHandle,
 }
 
 
-#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLConnect(HDBC ConnectionHandle,
 		   SQLCHAR *ServerName, SQLSMALLINT NameLength1,
@@ -293,6 +298,7 @@ SQLDisconnect(HDBC ConnectionHandle)
 	return ret;
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLExecDirect(HSTMT StatementHandle,
 			  SQLCHAR *StatementText, SQLINTEGER TextLength)
@@ -320,6 +326,7 @@ SQLExecDirect(HSTMT StatementHandle,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLExecute(HSTMT StatementHandle)
@@ -417,6 +424,7 @@ SQLFreeStmt(HSTMT StatementHandle,
 }
 
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLGetCursorName(HSTMT StatementHandle,
 				 SQLCHAR *CursorName, SQLSMALLINT BufferLength,
@@ -435,6 +443,7 @@ SQLGetCursorName(HSTMT StatementHandle,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLGetData(HSTMT StatementHandle,
@@ -478,6 +487,8 @@ SQLGetFunctions(HDBC ConnectionHandle,
 	LEAVE_CONN_CS(conn);
 	return ret;
 }
+
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLGetInfo(HDBC ConnectionHandle,
 		   SQLUSMALLINT InfoType, PTR InfoValue,
@@ -524,6 +535,7 @@ SQLGetTypeInfo(HSTMT StatementHandle,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLNumResultCols(HSTMT StatementHandle,
@@ -564,6 +576,7 @@ SQLParamData(HSTMT StatementHandle,
 	return ret;
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLPrepare(HSTMT StatementHandle,
 		   SQLCHAR *StatementText, SQLINTEGER TextLength)
@@ -589,6 +602,7 @@ SQLPrepare(HSTMT StatementHandle,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLPutData(HSTMT StatementHandle,
@@ -630,6 +644,7 @@ SQLRowCount(HSTMT StatementHandle,
 }
 
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLSetCursorName(HSTMT StatementHandle,
 				 SQLCHAR *CursorName, SQLSMALLINT NameLength)
@@ -646,6 +661,7 @@ SQLSetCursorName(HSTMT StatementHandle,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLSetParam(HSTMT StatementHandle,
@@ -666,6 +682,7 @@ SQLSetParam(HSTMT StatementHandle,
 }
 
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLSpecialColumns(HSTMT StatementHandle,
 				  SQLUSMALLINT IdentifierType, SQLCHAR *CatalogName,
@@ -949,6 +966,7 @@ SQLColumnPrivileges(HSTMT hstmt,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLDescribeParam(HSTMT hstmt,
@@ -1013,6 +1031,7 @@ SQLExtendedFetch(HSTMT hstmt,
 	return ret;
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLForeignKeys(HSTMT hstmt,
 			   SQLCHAR *szPkCatalogName,
@@ -1112,6 +1131,7 @@ SQLForeignKeys(HSTMT hstmt,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLMoreResults(HSTMT hstmt)
@@ -1132,6 +1152,7 @@ SQLMoreResults(HSTMT hstmt)
 	return ret;
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLNativeSql(HDBC hdbc,
 			 SQLCHAR *szSqlStrIn,
@@ -1152,6 +1173,7 @@ SQLNativeSql(HDBC hdbc,
 	LEAVE_CONN_CS(conn);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLNumParams(HSTMT hstmt,
@@ -1173,6 +1195,7 @@ SQLNumParams(HSTMT hstmt,
 	return ret;
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLPrimaryKeys(HSTMT hstmt,
 			   SQLCHAR *szCatalogName,
@@ -1391,6 +1414,7 @@ SQLProcedures(HSTMT hstmt,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLSetPos(HSTMT hstmt,
@@ -1414,6 +1438,7 @@ SQLSetPos(HSTMT hstmt,
 	return ret;
 }
 
+#ifndef	UNICODE_SUPPORTXX
 RETCODE		SQL_API
 SQLTablePrivileges(HSTMT hstmt,
 				   SQLCHAR *szCatalogName,
@@ -1483,6 +1508,7 @@ SQLTablePrivileges(HSTMT hstmt,
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
+#endif /* UNICODE_SUPPORTXX */
 
 RETCODE		SQL_API
 SQLBindParameter(HSTMT hstmt,
