@@ -17,6 +17,8 @@ extern "C" {
 #define	PODBC_NOT_SEARCH_PATTERN	1L
 #define	PODBC_SEARCH_PUBLIC_SCHEMA	(1L << 1)
 #define	PODBC_SEARCH_BY_IDS		(1L << 2)
+#define	PODBC_SHOW_OID_COLUMN		(1L << 3)
+#define	PODBC_ROW_VERSIONING		(1L << 4)
 /*	Internal flags for PGAPI_AllocStmt functions */
 #define	PODBC_EXTERNAL_STATEMENT	1L	/* visible to the driver manager */
 #define	PODBC_INHERIT_CONNECT_OPTIONS	(1L << 1)
@@ -295,7 +297,7 @@ RETCODE SQL_API PGAPI_GetStmtAttr(HSTMT StatementHandle,
 		SQLINTEGER Attribute, PTR Value,
 		SQLINTEGER BufferLength, SQLINTEGER *StringLength);
 
-/* Driver-specific connection attributes, for SQLSetConnectAttr() */
+/* Driver-specific connection attributes, for SQLSet/GetConnectAttr() */
 enum {
 	SQL_ATTR_PGOPT_DEBUG = 65536
 	,SQL_ATTR_PGOPT_COMMLOG = 65537
@@ -303,6 +305,13 @@ enum {
 	,SQL_ATTR_PGOPT_USE_DECLAREFETCH = 65539
 	,SQL_ATTR_PGOPT_SERVER_SIDE_PREPARE = 65540
 	,SQL_ATTR_PGOPT_FETCH = 65541
+	,SQL_ATTR_PGOPT_UNKNOWNSIZES = 65542
+	,SQL_ATTR_PGOPT_TEXTASLONGVARCHAR = 65543
+	,SQL_ATTR_PGOPT_UNKNOWNSASLONGVARCHAR = 65544
+	,SQL_ATTR_PGOPT_BOOLSASCHAR = 65545
+	,SQL_ATTR_PGOPT_MAXVARCHARSIZE = 65546
+	,SQL_ATTR_PGOPT_MAXLONGVARCHARSIZE = 65547
+	,SQL_ATTR_PGOPT_WCSDEBUG = 65548
 };
 RETCODE SQL_API PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 			SQLINTEGER Attribute, PTR Value,
