@@ -265,6 +265,16 @@ extern int posix_snprintf(char *buf, size_t size, const char *format, ...);
 
 #define DRIVER_ODBC_VER				"03.51"
 
+#ifndef	UNICODE_SUPPORT
+#undef	SQL_WCHAR
+#undef	SQL_WVARCHAR
+#undef	SQL_WLONGVARCHAR
+#undef	SQL_C_WCHAR
+#define	SQL_WCHAR	SQL_WCHAR_IS_INHIBITED
+#define	SQL_WVARCHAR	SQL_WVARCHAR_IS_INHIBITED
+#define	SQL_WLONGVARCHAR	SQL_WLONGVARCHAR_IS_INHIBITED
+#define	SQL_C_WCHAR	SQL_C_WCHAR_IS_INHIBITED
+#endif
 #ifndef DBMS_NAME
 #ifdef	UNICODE_SUPPORT
 #define DBMS_NAME				DBMS_NAME_UNICODE
@@ -579,6 +589,7 @@ typedef struct
 	signed char	ignore_round_trip_time;
 	signed char	disable_keepalive;
 	signed char	gssauth_use_gssapi;
+	signed char	wcs_debug;
 	UInt4		extra_opts;
 	Int4		keepalive_idle;
 	Int4		keepalive_interval;
@@ -609,11 +620,19 @@ const pthread_mutexattr_t *getMutexAttr(void);
 #endif /* POSIX_THREADMUTEX_SUPPORT */
 #ifdef	UNICODE_SUPPORT
 #define WCLEN sizeof(SQLWCHAR)
+enum {
+	WCSTYPE_UNKNOWN
+	,WCSTYPE_UTF16_LE
+	,WCSTYPE_UTF32_LE
+	};
 SQLULEN	ucs2strlen(const SQLWCHAR *ucs2str);
 char	*ucs2_to_utf8(const SQLWCHAR *ucs2str, SQLLEN ilen, SQLLEN *olen, BOOL tolower);
 SQLULEN	utf8_to_ucs2_lf(const char * utf8str, SQLLEN ilen, BOOL lfconv, SQLWCHAR *ucs2str, SQLULEN buflen, BOOL errcheck);
-int	msgtowstr(const char *, int, LPWSTR, int);
-int	wstrtomsg(const LPWSTR, int, char *, int);
+int	get_wcstype(void);
+int	msgtowstr(const char *, wchar_t *, int);
+int	wstrtomsg(const wchar_t *, char *, int);
+char	*wcs_to_utf8(const wchar_t *wcsstr, SQLLEN ilen, SQLLEN *olen, BOOL tolower);
+SQLULEN	utf8_to_wcs_lf(const char * utf8str, SQLLEN ilen, BOOL lfconv, wchar_t *wcsstr, SQLULEN buflen, BOOL errcheck);
 #define	utf8_to_ucs2(utf8str, ilen, ucs2str, buflen) utf8_to_ucs2_lf(utf8str, ilen, FALSE, ucs2str, buflen, FALSE)
 #endif /* UNICODE_SUPPORT */
 
@@ -625,6 +644,7 @@ int	wstrtomsg(const LPWSTR, int, char *, int);
 CSTR	NULL_STRING = "";
 CSTR	PRINT_NULL = "(null)";
 #define OID_NAME "oid"
+#define XMIN_NAME "xmin"
 #define TABLEOID_NAME "tableoid"
 
 #ifdef	__cplusplus
