@@ -60,9 +60,9 @@ QR_set_cache_size(QResultClass *self, SQLLEN cache_size)
 
 
 void
-QR_set_rowset_size(QResultClass *self, Int4 rowset_size)
+QR_set_reqsize(QResultClass *self, Int4 reqsize)
 {
-	self->rowset_size_include_ommitted = rowset_size;
+	self->rowset_size_include_ommitted = reqsize;
 }
 
 void
@@ -697,6 +697,8 @@ inolog("!!%p->cursTup=%d total_read=%d\n", self, self->cursTuple, self->num_tota
 	 */
 	QR_set_command(self, PQcmdStatus(*pgres));
 	QR_set_cursor(self, cursor);
+	if (NULL == cursor)
+		QR_set_reached_eof(self);
 	return TRUE;
 }
 
@@ -889,6 +891,43 @@ static SQLLEN enlargeKeyCache(QResultClass *self, SQLLEN add_size, const char *m
 	return alloc;
 }
 
+SQLLEN	QR_move_cursor_to_last(QResultClass *self, StatementClass *stmt)
+{
+	char		movecmd[64];
+	QResultClass	*res;
+	SQLLEN		moved;
+	ConnectionClass	*conn = SC_get_conn(stmt);
+
+	if (!QR_get_cursor(self))
+		return 0;
+	if (QR_once_reached_eof(self) &&
+	    self->cursTuple >= self->num_total_read)
+		return 0;
+	snprintf(movecmd, sizeof(movecmd),
+		 "move all in \"%s\"", QR_get_cursor(self));
+	res = CC_send_query(conn, movecmd, NULL, 0, stmt);
+	if (!QR_command_maybe_successful(res))
+	{
+		QR_Destructor(res);
+		SC_set_error(stmt, STMT_EXEC_ERROR, "move error occured", __FUNCTION__);
+		return (-1);
+	}
+	moved = (-1);
+	if (sscanf(res->command, "MOVE " FORMAT_ULEN, &moved) > 0)
+	{
+		moved++;
+		self->cursTuple += moved;
+		if (!QR_once_reached_eof(self))
+		{
+			self->num_total_read = self->cursTuple;
+			QR_set_reached_eof(self);
+		}
+	}
+	QR_Destructor(res);
+
+	return	moved;
+}
+
 /*	This function is called by fetch_tuples() AND SQLFetch() */
 int
 QR_next_tuple(QResultClass *self, StatementClass *stmt)
@@ -911,7 +950,6 @@ QR_next_tuple(QResultClass *self, StatementClass *stmt)
 	QueryInfo	qi;
 	ConnectionClass	*conn;
 	ConnInfo   *ci;
-	BOOL		internally_invoked = FALSE;
 	BOOL		reached_eof_now = FALSE, curr_eof; /* detecting EOF is pretty important */
 
 inolog("Oh %p->fetch_number=%d\n", self, self->fetch_number);
@@ -920,7 +958,12 @@ inolog("in total_read=%d cursT=%d currT=%d ad=%d total=%d rowsetSize=%d\n", self
 	num_total_rows = QR_get_num_total_tuples(self);
 	conn = QR_get_conn(self);
 	curr_eof = FALSE;
-	req_size = self->rowset_size_include_ommitted;
+	req_size = QR_get_reqsize(self);
+	/* Determine the optimum cache size.  */
+	ci = &(conn->connInfo);
+	fetch_size = ci->drivers.fetch_max;
+	if ((Int4)req_size > fetch_size)
+		fetch_size = req_size;
 	if (QR_once_reached_eof(self) && self->cursTuple >= (Int4) QR_get_num_total_read(self))
 		curr_eof = TRUE;
 #define	return	DONT_CALL_RETURN_FROM_HERE???
@@ -935,9 +978,9 @@ inolog("in total_read=%d cursT=%d currT=%d ad=%d total=%d rowsetSize=%d\n", self
 		movement = self->move_offset;
 		if (QR_is_moving_backward(self))
 		{
-			if (self->cache_size > req_size)
+			if (fetch_size > req_size)
 			{
-				SQLLEN	incr_move = self->cache_size - (req_size < 0 ? 1 : req_size);
+				SQLLEN	incr_move = fetch_size - (req_size < 0 ? 1 : req_size);
 
 				movement += incr_move;
 				if (movement > (UInt4)(self->cursTuple + 1))
@@ -991,69 +1034,6 @@ inolog("moved=%d ? " FORMAT_ULEN "\n", moved, movement);
 						QR_set_reached_eof(self);
 					}
 				}
-				if (QR_is_moving_from_the_last(self))  /* in case of FETCH LAST */
-				{
-					SQLULEN	bmovement, mback;
-					SQLLEN	rowset_start = self->cursTuple + 1, back_offset, backpt;
-inolog("FETCH LAST case\n");
-					if (getNthValid(self, QR_get_num_total_tuples(self) - 1, SQL_FETCH_PRIOR, self->move_offset, &backpt) < 0)
-					{
-						/* the rowset_start is on BOF */
-						self->tupleField = NULL;
-						SC_set_rowset_start(stmt, -1, TRUE);
-						stmt->currTuple = -1;
-						QR_Destructor(mres);
-						RETURN(-1)
-					}
-					back_offset = QR_get_num_total_tuples(self) - backpt;
-inolog("back_offset=%d and move_offset=%d\n", back_offset, self->move_offset);
-					if (back_offset + 1 > (Int4) self->ad_count)
-					{
-						bmovement = back_offset + 1 - self->ad_count;
-						snprintf(movecmd, sizeof(movecmd),
-								 "move backward " FORMAT_ULEN " in \"%s\"",
-								 bmovement, QR_get_cursor(self));
-						QR_Destructor(mres);
-						mres = CC_send_query(conn, movecmd, NULL, 0, stmt);
-						if (!QR_command_maybe_successful(mres))
-						{
-							QR_Destructor(mres);
-							SC_set_error(stmt, STMT_EXEC_ERROR, "move error occured", func);
-							RETURN(-1)
-						}
-
-						if (sscanf(mres->command, "MOVE " FORMAT_ULEN, &mback) > 0)
-						{
-							if (mback < bmovement)
-								mback++;
-							if (moved < mback)
-							{
-								QR_set_move_backward(self);
-								mback -= moved;
-								moved = mback;
-								self->move_offset = moved;
-								rowset_start = self->cursTuple - moved + 1;
-							}
-							else
-							{
-								QR_set_move_forward(self);
-								moved-= mback;
-								self->move_offset = moved;
-								rowset_start = self->cursTuple + moved + 1;
-							}
-						}
-					}
-					else
-					{
-						QR_set_move_forward(self);
-						self->move_offset = moved + self->ad_count - back_offset - 1;
-						rowset_start = self->cursTuple +self->move_offset + 1;
-						/* adjust move_offset */
-						/*** self->move_offset++; ***/
-					}
-					SC_set_rowset_start(stmt, rowset_start, TRUE); /* affects the result's rowset_start but it is reset immediately ... */
-					stmt->currTuple = RowIdx2GIdx(-1, stmt);
-				}
 			}
 		}
 		/* ... by the following call */
@@ -1111,7 +1091,6 @@ inolog("tupleField=%p\n", self->tupleField);
 	 */
 	self->tupleField = NULL;
 
-	ci = &(conn->connInfo);
 	if (!QR_get_cursor(self))
 	{
 		mylog("%s: ALL_ROWS: done, fcount = %d, fetch_number = %d\n", func, QR_get_num_total_tuples(self), fetch_number);
@@ -1126,17 +1105,6 @@ inolog("tupleField=%p\n", self->tupleField);
 		TupleField *tuple = self->backend_tuples;
 
 		/* not a correction */
-		/* Determine the optimum cache size.  */
-		if (ci->drivers.fetch_max % req_size == 0)
-			fetch_size = ci->drivers.fetch_max;
-		else if ((Int4)req_size < ci->drivers.fetch_max)
-		{
-			/*fetch_size = (ci->drivers.fetch_max / req_size + 1) * req_size;*/
-			fetch_size = (ci->drivers.fetch_max / req_size) * req_size;
-		}
-		else
-			fetch_size = req_size;
-
 		self->cache_size = fetch_size;
 		/* clear obsolete tuples */
 inolog("clear obsolete %d tuples\n", num_backend_rows);
@@ -1177,7 +1145,11 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 
 	mylog("%s: sending actual fetch (%d) query '%s'\n", func, fetch_size, fetch);
 	if (!boundary_adjusted)
+	{
 		QR_set_num_cached_rows(self, 0);
+		QR_set_rowstart_in_cache(self, offset);
+	}
+	num_rows_in = self->num_cached_rows;
 
 	/* don't read ahead for the next tuple (self) ! */
 	qi.row_size = self->cache_size;
@@ -1191,13 +1163,9 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 			QR_set_message(self, "Error fetching next group.");
 		RETURN(FALSE)
 	}
-	internally_invoked = TRUE;
 	cur_fetch = 0;
 
-	QR_set_rowstart_in_cache(self, offset);
-
 	self->tupleField = NULL;
-	num_rows_in = self->num_cached_rows;
 
 	curr_eof = reached_eof_now = (QR_once_reached_eof(self) && self->cursTuple >= (Int4)self->num_total_read);
 inolog("reached_eof_now=%d\n", reached_eof_now);
@@ -1205,6 +1173,7 @@ inolog("reached_eof_now=%d\n", reached_eof_now);
 	mylog("_%s: PGresult: fetch_total = %d & this_fetch = %d\n", func, self->num_total_read, self->num_cached_rows);
 	mylog("_%s: PGresult: cursTuple = %d, offset = %d\n", func, self->cursTuple, offset);
 
+	cur_fetch = self->num_cached_rows - num_rows_in;
 	if (!ret)
 		RETURN(ret)
 
@@ -1222,9 +1191,9 @@ inolog("reached_eof_now=%d\n", reached_eof_now);
 				SQLLEN	add_size;
 				TupleField	*tuple, *added_tuple;
 
-				if (curr_eof)
+				start_idx = CacheIdx2GIdx(offset, stmt, self) - self->num_total_read;
+				if (curr_eof && start_idx >= 0)
 				{
-					start_idx = CacheIdx2GIdx(offset, stmt, self) - self->num_total_read;
 					add_size = self->ad_count - start_idx;
 					if (0 == num_backend_rows)
 					{
@@ -1240,11 +1209,9 @@ inolog("reached_eof_now=%d\n", reached_eof_now);
 				}
 				if (add_size > fetch_size - cur_fetch)
 					add_size = fetch_size - cur_fetch;
-inolog("will add %d added_tuples from %d and select the %dth added tuple\n", add_size, start_idx, offset - num_backend_rows + start_idx);
-				if (add_size > fetch_size - cur_fetch)
-					add_size = fetch_size - cur_fetch;
 				else if (add_size < 0)
 					add_size = 0;
+inolog("will add %d added_tuples from %d and select the %dth added tuple %d\n", add_size, start_idx, offset - num_backend_rows + start_idx, cur_fetch);
 				if (enlargeKeyCache(self, add_size, "Out of memory while adding tuples") < 0)
 					RETURN(FALSE)
 				/* append the KeySet info first */
@@ -1276,27 +1243,32 @@ inolog("will add %d added_tuples from %d and select the %dth added tuple\n", add
 	 If the cursor operation was invoked inside this function,
 	 we have to set the status bits here.
 	*/
-	if (internally_invoked && self->keyset && (self->dl_count > 0 || self->up_count > 0))
+	if (self->keyset && (self->dl_count > 0 || self->up_count > 0))
 	{
 		SQLLEN	i, lf;
-		SQLLEN	lidx, hidx;
+		SQLLEN	lidx, hidx, lkidx, hkidx;
 		SQLLEN	*deleted = self->deleted, *updated = self->updated;
 
 		num_backend_rows = QR_get_num_cached_tuples(self);
+		lidx = CacheIdx2GIdx(num_rows_in, stmt, self);
+		hidx = CacheIdx2GIdx(num_backend_rows, stmt, self);
+		lkidx = GIdx2KResIdx(lidx, stmt, self);
+		hkidx = GIdx2KResIdx(hidx, stmt, self);
 		/* For simplicty, use CURS_NEEDS_REREAD bit to mark the row */
-		for (i = num_rows_in; i < num_backend_rows; i++)
+		for (i = lkidx; i < hkidx; i++)
 			self->keyset[i].status |= CURS_NEEDS_REREAD;
-		hidx = RowIdx2GIdx(num_backend_rows, stmt);
-		lidx = hidx - num_backend_rows;
 		/* deleted info */
 		for (i = 0; i < self->dl_count && hidx > deleted[i]; i++)
 		{
 			if (lidx <= deleted[i])
 			{
-				lf = num_backend_rows - hidx + deleted[i];
-				self->keyset[lf].status = self->deleted_keyset[i].status;
-				/* mark the row off */
-				self->keyset[lf].status &= (~CURS_NEEDS_REREAD);
+				lf = GIdx2KResIdx(deleted[i], stmt, self);
+				if (lf >= 0 && lf < self->num_cached_keys)
+				{
+					self->keyset[lf].status = self->deleted_keyset[i].status;
+					/* mark the row off */
+					self->keyset[lf].status &= (~CURS_NEEDS_REREAD);
+				}
 			}
 		}
 		for (i = self->up_count - 1; i >= 0; i--)
@@ -1304,7 +1276,7 @@ inolog("will add %d added_tuples from %d and select the %dth added tuple\n", add
 			if (hidx > updated[i] &&
 			    lidx <= updated[i])
 			{
-				lf = num_backend_rows - hidx + updated[i];
+				lf = GIdx2KResIdx(updated[i], stmt, self);
 				/* in case the row is marked off */
 				if (0 == (self->keyset[lf].status & CURS_NEEDS_REREAD))
 					continue;
